# Copyright (c) 2009 Daniel Holth <dholth@fastmail.fm>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import

__all__ = ['PBKDF2PasswordManager']

import os
from base64 import urlsafe_b64encode, urlsafe_b64decode
from cryptacular.core import check_unicode
import cryptacular.core
try: # pragma NO COVERAGE
    import M2Crypto.EVP
    _pbkdf2 = M2Crypto.EVP.pbkdf2
except (ImportError, AttributeError): # pragma NO COVERAGE
    from pbkdf2 import PBKDF2
    def _pbkdf2(password, salt, rounds, keylen):
        return PBKDF2(password, salt, rounds).read(keylen)

class PBKDF2PasswordManager(object):

    SCHEME = "PBKDF2"
    PREFIX = "$p5k2$"
    ROUNDS = 1<<12

    def encode(self, password, salt=None, rounds=None, keylen=20):
        if salt is None:
            salt = os.urandom(16)
        rounds = rounds or self.ROUNDS
        password = check_unicode(password)
        key = _pbkdf2(password, salt, rounds, keylen)
        hash =  self.PREFIX.encode('iso8859-1') + \
                ('%x' % rounds).encode('iso8859-1') + b'$' + \
                urlsafe_b64encode(salt) + b'$' + \
                urlsafe_b64encode(key)
        return hash.decode('utf-8')

    def check(self, encoded, password):
        encoded = check_unicode(encoded)
        if not self.match(encoded):
            return False
        iter, salt, key = encoded[len(self.PREFIX):].split('$')
        iter = int(iter, 16)
        salt = urlsafe_b64decode(salt.encode('utf-8'))
        keylen = len(urlsafe_b64decode(key.encode('utf-8')))
        hash = self.encode(password, salt, iter, keylen)
        return cryptacular.core._cmp(hash, encoded)

    def match(self, encoded):
        """True if encoded appears to match this scheme."""
        encoded = check_unicode(encoded)
        return encoded.startswith(self.PREFIX)
 



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from nose.tools import eq_, raises, assert_false, assert_true, assert_not_equal
import cryptacular.bcrypt
from cryptacular.bcrypt import BCRYPTPasswordManager

try:
    unicode
except NameError:
    unicode = str

class TestBCRYPTPasswordManager(object):
    snowpass = "hashy the \N{SNOWMAN}"

    def setup(self):
        self.manager = BCRYPTPasswordManager()

    @raises(TypeError)
    def test_None1(self):
        self.manager.encode(None)

    @raises(TypeError)
    def test_None2(self):
        self.manager.check(None, 'xyzzy')

    @raises(TypeError)
    def test_None3(self):
        hash = self.manager.encode('xyzzy')
        self.manager.check(hash, None)

    def test_badhash(self):
        eq_(self.manager.check('$p5k2$400$ZxK4ZBJCfQg=$kBpklVI9kA13kP32HMZL0rloQ1M=', self.snowpass), False)

    @raises(ValueError)
    def test_shorthash(self):
        manager = BCRYPTPasswordManager()
        def match(hash):
            return True
        manager.match = match
        short_hash = manager.encode(self.snowpass)[:28]
        assert_true(manager.match(short_hash))
        manager.check(short_hash, self.snowpass)

    @raises(ValueError)
    def test_too_few_rounds(self):
        self.manager.encode(self.snowpass, rounds=1)

    @raises(ValueError)
    def test_too_many_rounds(self):
        self.manager.encode(self.snowpass, rounds=100)

    def test_emptypass(self):
        self.manager.encode('')

    def test_general(self):
        manager = self.manager
        hash = manager.encode(self.snowpass)
        eq_(manager.match(hash), True)
        eq_(len(hash), 60)
        assert_true(manager.check(hash, self.snowpass))
        password = "xyzzy"
        hash = manager.encode(password)
        assert_true(manager.check(hash, password))
        assert_true(manager.check(unicode(hash), password))
        assert_false(manager.check(password, password))
        assert_not_equal(manager.encode(password), manager.encode(password))
        hash = manager.encode(password, rounds=4)
        assert_true(manager.check(hash, password))
    
    @raises(ValueError)
    def test_fail_1(self):
        def return_none(*args): return None
        bcrypt = BCRYPTPasswordManager()
        bcrypt.crypt_gensalt_rn = return_none
        bcrypt.encode('foo')

    @raises(ValueError)
    def test_fail_2(self):
        def return_none(*args): return None
        bcrypt = BCRYPTPasswordManager()        
        bcrypt.crypt_rn = return_none
        bcrypt.encode('foo')

    @raises(ValueError)
    def test_fail_3(self):
        def return_none(*args): return None
        bcrypt = BCRYPTPasswordManager()
        pw = bcrypt.encode('foobar')
        bcrypt.crypt_rn = return_none
        bcrypt.check(pw, 'foo')



from __future__ import unicode_literals
from nose.tools import eq_, raises, assert_false, assert_true, assert_not_equal
from cryptacular.crypt import *

class TestCRYPTPasswordManager(object):
    snowpass = "hashy the \N{SNOWMAN}"

    PREFIX = OLDCRYPT

    def setup(self):
        self.manager = CRYPTPasswordManager(self.PREFIX)

    @raises(TypeError)
    def test_None1(self):
        self.manager.encode(None)

    @raises(TypeError)
    def test_None2(self):
        self.manager.check(None, 'xyzzy')

    @raises(TypeError)
    def test_None3(self):
        hash = self.manager.encode('xyzzy')
        self.manager.check(hash, None)

    def test_badhash(self):
        eq_(self.manager.check('$p5k2$400$ZxK4ZBJCfQg=$kBpklVI9kA13kP32HMZL0rloQ1M=', self.snowpass), False)

    def test_shorthash(self):
        manager = self.manager
        def match(hash):
            return True
        manager.match = match
        short_hash = manager.encode(self.snowpass)[:11]
        assert_true(manager.match(short_hash))
        assert_false(manager.check(short_hash, self.snowpass))

    def test_emptypass(self):
        self.manager.encode('')

    def test_general(self):
        manager = self.manager
        hash = manager.encode(self.snowpass)
        eq_(manager.match(hash), True)
        assert hash.startswith(self.PREFIX)
        assert_true(manager.check(hash, self.snowpass))
        password = "xyzzy"
        hash = manager.encode(password)
        assert_true(manager.check(hash, password))
        assert_false(manager.check(password, password))
        assert_not_equal(manager.encode(password), manager.encode(password))

available = CRYPTPasswordManager('').available

if available(BCRYPT):
    class TestCPM_BCRYPT(TestCRYPTPasswordManager):
        PREFIX = BCRYPT

if available(MD5CRYPT):
    class TestCPM_MD5CRYPT(TestCRYPTPasswordManager):
        PREFIX = MD5CRYPT

if available(SHA256CRYPT):
    class TestCPM_SHA256CRYPT(TestCRYPTPasswordManager):
        PREFIX = SHA256CRYPT

if available(SHA512CRYPT):
    class TestCPM_SHA512CRYPT(TestCRYPTPasswordManager):
        PREFIX = SHA512CRYPT

@raises(NotImplementedError)
def test_bogocrypt():
    CRYPTPasswordManager('$bogo$')

@raises(NotImplementedError)
def test_oddcrypt():
    """crypt.crypt with empty prefix returns hash != 13 characters?"""
    class BCPM(CRYPTPasswordManager):
        _crypt = lambda x, y, z: '4' * 14
    BCPM('')
